package PVE::HTMLForm;

use strict;
use HTML::Entities;
use PVE::I18N;

sub new {
    my ($type, $formdata, $name) = @_;
    my $self = {};
    
    $self->{formdata} = $formdata;
    $self->{elements} = 0;# internal element counter
    $self->{submit} = 0;			
    $self->{name} = $name ? $name : "frm";

    bless($self, $type);
	
    $self->{action} = $self->{formdata}->{"form_$self->{name}_submit"};

    $self->postaction;
    
    return $self;
}

sub postaction {
    my ($self) = @_;

    my ($key, %d);
	
    # Rebuild Date for IP, Bool and Time fields
    foreach $key (sort keys (%{$self->{formdata}})) {
	if ($key =~ m/ip_[0-9]+_(.*)/) {
	    if (!(exists ($d{$1})))  {
		$d{$1}=1;
		defined ($self->{formdata}->{"ip_0_$1"}) || 
		    ($self->{formdata}->{"ip_0_$1"} = '0');
		defined ($self->{formdata}->{"ip_1_$1"}) || 
		    ($self->{formdata}->{"ip_1_$1"} = '0');
		defined ($self->{formdata}->{"ip_2_$1"}) || 
		    ($self->{formdata}->{"ip_2_$1"} = '0');
		defined ($self->{formdata}->{"ip_3_$1"}) || 
		    ($self->{formdata}->{"ip_3_$1"} = '0');
		$self->{formdata}->{"$1"} = 	
		    $self->{formdata}->{"ip_0_$1"} . "." .
		    $self->{formdata}->{"ip_1_$1"} . "." .
		    $self->{formdata}->{"ip_2_$1"} . "." . 
		    $self->{formdata}->{"ip_3_$1"};
	    }	
	}
	if ($key =~ m/time_[0-9]+_(.*)/) {
	    if (!(exists ($d{$1})))  {
		$d{$1}=1;
		defined ($self->{formdata}->{"time_0_$1"}) || 
		    ($self->{formdata}->{"time_0_$1"} = '00');
		defined ($self->{formdata}->{"time_1_$1"}) || 
		    ($self->{formdata}->{"time_1_$1"} = '00');
		$self->{formdata}->{"$1"} = 	
		    $self->{formdata}->{"time_0_$1"} . ":" .
		    $self->{formdata}->{"time_1_$1"};
	    }
	}
	if ($key =~ m/cb_n_(.*)/) {
	    if (!(exists ($d{$1})))  {
		my $name = $1;
		$d{$name} = 1;
		my $val = "";
		my $tmp;
		foreach my $k (keys (%{$self->{formdata}})) {
		    if ($k =~ m/cb__(\w+)_$name/) {
			$tmp->{$1} = 1;
		    }
		}
		foreach my $k (keys (%$tmp)) {
		    $val .= " " if $val;
		    $val .= $k;
	        }
		$self->{formdata}->{"$1"} = $val;
	    }
	}
	if ($key =~ m/bool_n_(.*)/) {
	    if (!(exists ($d{$1})))  {
		$d{$1}=1;
		my $val = "0";
		if ($self->{formdata}->{"bool_$1"}) {
		    $val = "1";
		}
		$self->{formdata}->{"$1"} = $val;
	    }
	}
    }
}

sub action {
    my ($self) = @_;
    return $self->{action};
}

sub viewonly_element {
   my $text = shift;
   my $encvalue = encode_entities ($text);
   return "<input disabled class=normal readonly type='text' value='$encvalue'>";
}

sub create_element {
    my ($self, $name, $type, $value, $opt, $width) = @_;

    my $out = '';

    my $class = 'normal';

    my $encvalue = encode_entities ($value);

    my $widthstr = $width ? "style='width:${width}px;'" : '';

    # normal text
    if ($type eq "text") {
	$out .= "<input $widthstr class='$class' type='text' name=$name value='$encvalue'/>";
    }
    # read only text
    elsif ($type eq "rotext") {
	$out .= "<input type='hidden' id='$name' name=$name value='$encvalue'/>";
	$out .= "<input readonly disabled $widthstr class='$class' type='text' name='${name}_ro' value='$encvalue'/>";
    }
    # server time
    elsif ($type eq "rotime") {
	my $uid = PVE::HTMLControls::get_uid('mytimer'); 
	$out .= "<div class='bool input' id='$uid'>$encvalue</div>";
	$out .= PVE::HTMLControls::create_time_viewer ($uid);
    }
    # time of day
    elsif ($type eq "time") {
    	my @tmp = split(/:/, $value);
    	for my $i (0..1) {
	    $out .= "<input type=text name=time_${i}_$name class='$class time' value='$tmp[$i]' />";
	    $out .= " : " if !$i;
        }
    }
    # password
    elsif ($type eq "password") {
	$out .= "<input $widthstr class='$class' type='password' name=$name value='$encvalue'/>";
    }
    # number
    elsif ($type eq "number") {
	$out .= "<input $widthstr class='$class' type='text' id='$name' name='$name' value='$encvalue'/>";
    }
    # float
    elsif ($type eq "float") {
	$out .= "<input $widthstr class='$class' type='text' id='$name' name='$name' value='$encvalue'/>";
    }
    # boolean value (0, 1)
    elsif ($type eq "bool" || $type eq "robool" || $type eq 'dynamicbool' || $type eq "nbool") {
	my $checked = $value ? 'checked' : '';
	my $id = "bool_$name";

	$out .= "<div style='padding:1px 0;margin:0px;'class='$class bool' align=center>";
	if ($type eq 'dynamicbool') {  
	    $out .= "<input type='checkbox' name='$id' id='$id' $checked " . 
		"onClick='javascript:pve_form_save(\"$self->{name}\", \"post\");'/>";
	} else {
	    if ($type eq "robool") {
		$out .= "<input type='checkbox' disabled name='${id}_ro' $checked/>";
		$out .= "<input type='hidden' name='$id' id='$id' $checked/>";
	    } else {
		$out .= "<input type='checkbox' name='$id' id='$id' $checked/>";
	    }
	}
	$out .= "</div>";
	$out .= "<input type='hidden' name='bool_n_$name' id='bool_n_$name' value='1'/>";
    }
    # dropdown
    elsif (($type eq "dropdown")||($type eq "dynamicdropdown")) {
	if ($type eq "dynamicdropdown") {  
	    $out .= "<select $widthstr id='$name' name='$name' class='$class' onChange='javascript:pve_form_save(\"$self->{name}\", \"post\");'>";
	} else {	
	    $out .= "<select $widthstr id='$name' name='$name' class='$class'>";
	}
	if ($opt) {
	    my @element = @$opt;
	    for my $i (0 .. $#element) {
		my ($ev, $et);
		if (ref ($element[$i]) eq 'ARRAY') {
		    ($ev, $et) = ($element[$i][0], $element[$i][1]);
		} else {
		    $ev = $et = $element[$i];
		}
		my $val = ($ev eq $value) ? 'selected="selected"' : '';	 
		
		$out .= "<option value='$ev' $val>$et</option>";    
	    }		
	}
	$out .= "</select>";		
    }
    # ip address
    elsif ($type eq "ip") {
	my @tmp = split(/\./, $value);
		
	for my $i (0..3) {
	    my $id = "ip_${i}_$name";
	    my $nextid = $i < 3 ? "ip_" . ($i+1) . "_$name" : '';
	    $out .= <<__EOD;
<input class='$class ip' type='text' id='$id' name='$id' value='$tmp[$i]'
    onKeyUp="pve_form_validate_natural('$id', 255);" 
    onKeyDown="return pve_form_ip_keyfilter(event,'$id','$nextid');"
/>
__EOD
	    if ($i != 3) { 
		$out .= "&nbsp;<b style='font-family:ARIAL; " . 
		    "font-weight:bold; font-size:14px;'>.</b>&nbsp;&nbsp;";
	    }
	}

    }
    # file upload 
    elsif ($type eq "file") {
	# most wrowsers (firefox) ignore setting 'width', dont know how to fix
	$out .= "<input $widthstr class='$class' type='file' id='$name' name='$name'/>";
    }
    # hidden input
    elsif ($type eq "hidden") {
	if (defined ($value)) {
	    $out .= "<input type='hidden' id='$name' name=$name value='$encvalue'/>";
	}
    } 
    # Day of week
    elsif ($type eq "dow") {
	my $cl = { mon => '', tue => '', wed => '', thu => '',
		   fri => '', sat => '', sun => '' };
	foreach my $day (split (/\s+/, $value)) {
	    $cl->{$day} = 'checked';
	}
	my @dn = split (/\s+/, __("Mon Tue Wed Thu Fri Sat Sun"));
	my $w='15%';
	$out .= "<div class='bool inputbg stdbdshape lightcolbd'><table cellspacing=2 cellpadding=0>";
	$out .= "<tr align=center><td width='$w'>$dn[0]<td width='$w'>$dn[1]<td width='$w'>$dn[2]";
	$out .= "<td width='$w'>$dn[3]<td width='$w'>$dn[4]<td width='$w'>$dn[5]<td width='$w'>$dn[6]</tr>";
	$out .= "<tr align=center>";
	$out .= "<td><input type=checkbox $cl->{mon} name='$name' value=mon>";
	$out .= "<td><input type=checkbox $cl->{tue} name='$name' value=tue>";
	$out .= "<td><input type=checkbox $cl->{wed} name='$name' value=wed>";
	$out .= "<td><input type=checkbox $cl->{thu} name='$name' value=thu>";
	$out .= "<td><input type=checkbox $cl->{fri} name='$name' value=fri>";
	$out .= "<td><input type=checkbox $cl->{sat} name='$name' value=sat>";
	$out .= "<td><input type=checkbox $cl->{sun} name='$name' value=sun>";
	$out .= "</tr></table></div>";
    }
    else {
	die "unknown elemet type '$type'";
    }

    $out .= "\n";

    return $out;
}

sub create_cmdbutton {
    my ($self, $type, $text) = @_;

    my $href = "javascript:pve_form_save(\"$self->{name}\", \"$type\");";

    my $out = "<img alt='' style='vertical-align:text-bottom;width:15px; height:15px;' src='/images/tarrright.png'>&nbsp;";

    $text = $type if !$text;

    if ($type eq "save") {
	$text = __('save');
    } elsif ($type eq "search") {
	$text = __('search');
    } elsif ($type eq "upload") {
	$text = __('upload');
    } 

    $out .= "<a href='$href' class='frmsubmit'>$text</a>";

    return $out;
}

sub create_header {
    my ($self, $action) = @_;
    
    $action = '' if !$action;

    return "<form style='margin:0px;' id='$self->{name}' action='$action' method='POST' ENCTYPE='multipart/form-data' accept-charset='UTF-8'>";
}

sub create_footer {
    my $self = shift;

    my $out = $self->create_element("form_$self->{name}_submit", 'hidden', 'post');
    $out .= "</form>";

    return $out;
}

1;
