package PVE::Qemu;

use strict;
use IO::Dir;
use IO::File;
use PVE::Config;
use PVE::QemuServer;
use PVE::SafeSyslog;
use Time::HiRes qw (gettimeofday);

my $last_proc_pid_stat;

my $clock_ticks = POSIX::sysconf(&POSIX::_SC_CLK_TCK);

sub vmlist {

    my $res = {};

    my $list =  PVE::QemuServer::vzlist();
    my ($uptime) = PVE::Utils::read_proc_uptime();

    foreach my $veid (keys %$list) {
	my $fn = PVE::QemuServer::config_file ($veid);
	my $conf = PVE::Config::read_file ($fn);
     
	my $d = {};
	$d->{type} = 'qemu';
	$d->{nproc} = 0;
	$d->{pid} = $list->{$veid}->{pid};
	$d->{status} = $list->{$veid}->{pid} ? 'running' : 'stopped';
	$d->{ip} = '-';
	$d->{disk} = 0;
	$d->{cpus} = $conf->{smp} || 1;
	$d->{name} = $conf->{name} || "VM$veid";

	$d->{maxmem} = $conf->{memory} ? $conf->{memory} : 0; 

	$d->{maxdisk} = $conf->{disksize} ? int($conf->{disksize}*1024) : 0;

	$d->{uptime} = 0;
	$d->{pctcpu} = 0;
	$d->{relcpu} = 0;
	$d->{mem} = 0;

	$res->{"VEID_$veid"} = $d;

    }

    my $cpuinfo = PVE::Utils::get_cpu_info();
    my $cpucount = $cpuinfo->{cpus} || 1;
    my $ctime = gettimeofday;

    foreach my $veid (keys %$list) {

	my $d = $res->{"VEID_$veid"};
	my $pid = $d->{pid};
	next if !$pid;

	my $statstr;
	if (my $fh = IO::File->new ("/proc/$pid/stat", "r")) {
	    $statstr = <$fh>;
	    $fh->close;
	}
	next if !$statstr;

	my ($utime, $stime, $vsize, $rss, $starttime);
	if ($statstr =~ m/^$pid \(.*\) \S (-?\d+) -?\d+ -?\d+ -?\d+ -?\d+ \d+ \d+ \d+ \d+ \d+ (\d+) (\d+) (-?\d+) (-?\d+) -?\d+ -?\d+ -?\d+ 0 (\d+) (\d+) (-?\d+) \d+ \d+ \d+ \d+ \d+ \d+ \d+ \d+ \d+ \d+ \d+ \d+ \d+ -?\d+ -?\d+ \d+ \d+ \d+/) {
	    ($utime, $stime, $vsize, $rss, $starttime) = ($2, $3, $7, $8 * 4096, $6);
	} else {
	    next;
	}
 
	my $used = $utime + $stime;

	my $vcpus = $d->{cpus} > $cpucount ? $cpucount : $d->{cpus};

	$d->{uptime} = int ($uptime - ($starttime/100));

	if ($vsize) {
	    $d->{mem} = int (($rss/$vsize)*$d->{maxmem});
	}

	my $old = $last_proc_pid_stat->{$pid};
	if (!$old) {
	    $last_proc_pid_stat->{$pid} = { 
		time => $ctime, 
		used => $used,
		pctcpu => 0,
		relcpu => 0,
	    };
	    next;
	}

	my $dtime = ($ctime -  $old->{time}) * $cpucount * $clock_ticks;

	if ($dtime > 1000) {
	    my $dutime = $used -  $old->{used};

	    $d->{pctcpu} = int((($dutime)*100)/$dtime);
	    $d->{relcpu} = int (($d->{pctcpu} * $cpucount) / $vcpus);
	    $last_proc_pid_stat->{$pid} = {
		time => $ctime, 
		used => $used,
		pctcpu => $d->{pctcpu},
		relcpu => $d->{relcpu},
	    };
	} else {
	    $d->{pctcpu} = $old->{pctcpu};
	    $d->{relcpu} = $old->{relcpu};
	}
    }

    return $res;
}

1;
